// *****************************************************************************
// Arduino 101 data logger using TI SensorTag CC2650                          **
// Software to accompany the article:                                         **
// "Build an Arduino 101 Data Logger with the TI SensorTag"                   **
// *****************************************************************************
// This software is offered strictly as is with no warranties whatsoever.     **
// Use it at your own risk.                                                   **
// *****************************************************************************
// SensorTagDLv1.ino

#include <CurieBLE.h>
#include <Wire.h>
#include <SD.h>
#include <SPI.h>
#include "RTClib.h"

// real time clock on the data logger shield
RTC_PCF8523 rtc;

// constants
const byte red_LED = 7;   // L2 on SD card
const byte green_LED = 6; // L1 on SD card
const byte CS_pin = 10;   // SD chip select line of the SD card

// Global structure to hold the sensor data
struct DATA {
  float tem;     // HDC1000 temperature F
  float hum;     // HDC1000 relative humidity %RH
  float bptemp;  // BMP 280 die temperature F
  float bp;      // BMP 280 barometric pressure in hectoPascals (1 hPa = 100 Pa)
  float li;      // OPT3001 lux
  float temd;    // TMP007 die temperature F
  float temo;    // TMP007 object temperature F
};

typedef struct DATA DATA;
DATA SensorData;

//period defines the length of time between measurements in milliseconds
long period = 600000L; // 10 minutes
//long period = 60000L;   // 1 minute

File SDF;
char fname[] = "STDATA.CSV"; // Log File Name

DateTime now;  // RTC object
BLEDevice peripheral;

// BMP280
BLECharacteristic BPConCharacteristic;
BLECharacteristic BPValCharacteristic;
// OPT3001
BLECharacteristic OPTConCharacteristic;
BLECharacteristic OPTValCharacteristic;
// TMP007
BLECharacteristic IRTConCharacteristic;
BLECharacteristic IRTValCharacteristic;
// HDC1000
BLECharacteristic HUMConCharacteristic;
BLECharacteristic HUMValCharacteristic;


void setup() {
  pinMode(red_LED, OUTPUT);
  pinMode(green_LED, OUTPUT);
  pinMode(CS_pin, OUTPUT);
  Wire.begin();
  // make sure the rtc is up
  if (! rtc.begin()) {
    // Terminal error
    t_error();
  }

  // NOTE: the following code line sets the RTC to the date and time that this sketch
  // was compiled - if you set the RTC another way, and you have already set the time
  // then comment out the code line
  rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  // make sure the SD is up
  if (!SD.begin(CS_pin)) {
    // terminal error
    t_error();
    return;
  }
  else
  {
    // if SD card is initialized, check if the log file exists
    // and create it if not
    if (! SD.exists(fname)) {
      SDF = SD.open(fname, FILE_WRITE);
      SDF.close();
    }
  }

  // initialize the BLE hardware
  BLE.begin();
  BLE.scan();
}

void loop() {
  long lastMillis = 0;  // for period test
  long nowMillis = 0;   // for period test

  // check if a peripheral has been discovered
  peripheral = BLE.available();

  digitalWrite(red_LED, HIGH); // not connected status

  if (peripheral) {
    // discovered a peripheral - see if it is a SensorTag
    // NOTE: if there is more than one, change to search for MAC
    if (peripheral.localName() == "CC2650 SensorTag") {
      // stop scanning
      BLE.stopScan();
      // connect to the peripheral
      if (peripheral.connect()) {
        digitalWrite(red_LED, LOW);  // turn disconected status light off
        do_discovery(peripheral);
        subscribe_BP(peripheral);
        subscribe_OPT(peripheral);
        subscribe_IRT(peripheral);
        subscribe_HUM(peripheral);
      } else {
        // terminal error
        t_error();
      }
    }
    // main sensor read and write loop
    while (peripheral.connected()) {
      read_BP(peripheral);
      read_OPT(peripheral);
      read_IRT(peripheral);
      read_HUM(peripheral);
      write_data();
      lastMillis = millis();
      // stay here until the period is up
      while ( ( (nowMillis = millis() ) - lastMillis) < period) {
        // toggle green LED every 500 milliseconds
        while ( (millis() - nowMillis) < 500);
        // toggle the green LED
        digitalWrite(green_LED, !digitalRead(green_LED));
      }
    }
    // peripheral disconnected, start scanning again
    digitalWrite(red_LED, HIGH);  // signal unconnected status
    digitalWrite(green_LED, LOW);
    BLE.scan();
  }
}

// discover the peripheral's attributes that we want
void do_discovery(BLEDevice peripheral) {
  // discover peripheral attributes for the barometric pressure sensor
  if (peripheral.discoverAttributesByService("F000AA40-0451-4000-B000-000000000000")) {
    BPConCharacteristic = peripheral.characteristic("F000AA42-0451-4000-B000-000000000000");
    BPValCharacteristic = peripheral.characteristic("F000AA41-0451-4000-B000-000000000000");
  }
  else  {
    peripheral.disconnect();
    return;
  }
  // discover peripheral attributes for the optical sensor
  if (peripheral.discoverAttributesByService("F000AA70-0451-4000-B000-000000000000")) {
    OPTConCharacteristic = peripheral.characteristic("F000AA72-0451-4000-B000-000000000000");
    OPTValCharacteristic = peripheral.characteristic("F000AA71-0451-4000-B000-000000000000");
  }
  else  {
    peripheral.disconnect();
    return;
  }
  // discover peripheral attributes for the infrared sensor
  if (peripheral.discoverAttributesByService("F000AA00-0451-4000-B000-000000000000")) {
    IRTConCharacteristic = peripheral.characteristic("F000AA02-0451-4000-B000-000000000000");
    IRTValCharacteristic = peripheral.characteristic("F000AA01-0451-4000-B000-000000000000");
  }
  else  {
    peripheral.disconnect();
    return;
  }
  // discover peripheral attributes for the humidity sensor
  if (peripheral.discoverAttributesByService("F000AA20-0451-4000-B000-000000000000")) {
    HUMConCharacteristic = peripheral.characteristic("F000AA22-0451-4000-B000-000000000000");
    HUMValCharacteristic = peripheral.characteristic("F000AA21-0451-4000-B000-000000000000");
  }
  else  {
    peripheral.disconnect();
    return;
  }

}

// subscribe to each of the peripheral's services that we want
void subscribe_BP(BLEDevice peripheral)
{
  // subscribe to the barometric service
  if (!BPValCharacteristic) {
    peripheral.disconnect();
    return;
  } else if (!BPValCharacteristic.canSubscribe()) {
    peripheral.disconnect();
    return;
  } else if (!BPValCharacteristic.subscribe()) {
    peripheral.disconnect();
    return;
  } else {
    delay(1000);
  }
}

void subscribe_OPT(BLEDevice peripheral) {
  // subscribe to the luxometer service
  if (!OPTValCharacteristic) {
    peripheral.disconnect();
    return;
  } else if (!OPTValCharacteristic.canSubscribe()) {
    peripheral.disconnect();
    return;
  } else if (!OPTValCharacteristic.subscribe()) {
    peripheral.disconnect();
    return;
  } else {
    delay(1000);
  }
}

void subscribe_IRT(BLEDevice peripheral) {
  // subscribe to the infrared service
  if (!IRTValCharacteristic) {
    peripheral.disconnect();
    return;
  } else if (!IRTValCharacteristic.canSubscribe()) {
    peripheral.disconnect();
    return;
  } else if (!IRTValCharacteristic.subscribe()) {
    peripheral.disconnect();
    return;
  } else {
    delay(1000);
  }
}

void subscribe_HUM(BLEDevice peripheral) {
  // subscribe to the humidity service
  if (!HUMValCharacteristic) {
    peripheral.disconnect();
    return;
  } else if (!HUMValCharacteristic.canSubscribe()) {
    peripheral.disconnect();
    return;
  } else if (!HUMValCharacteristic.subscribe()) {
    peripheral.disconnect();
    return;
  } else {
    delay(1000);
  }
}

// Sensor Reads
void read_BP(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    BPConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned long rawbptemp = (BPValCharacteristic[2] * 65536) + (BPValCharacteristic[1] * 256) + BPValCharacteristic[0];
    unsigned int rawbp = (BPValCharacteristic[5] * 65536) + (BPValCharacteristic[4] * 256) + BPValCharacteristic[3];
    BPConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate temperature and pressure final values
    float bptemp = ((double)rawbptemp / 100.0);
    bptemp = ((bptemp * 9.0) / 5.0) + 32.0; // convert to F - comment out to leave at C
    float bp = ((double)rawbp / 100.0);
    // save into the structure
    SensorData.bp = bp;
    SensorData.bptemp = bptemp;
  }
  else {
    // insert not connected error signal here if you want
  }
}

void read_OPT(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    // while the peripheral is connected, turn the optic sensor on by writing a '1' to the characteristic
    OPTConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned int rawlux = (OPTValCharacteristic[0]) + (OPTValCharacteristic[1] * 256);
    OPTConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate lux final value
    unsigned int m = rawlux & 0x0FFF;
    unsigned int e = (rawlux & 0xF000) >> 12;
    float lux = (m * (0.01 * pow(2.0, e)));
    // save into the structure
    SensorData.li = lux;
  }
  else {
    // insert not connected error signal here if you want
  }
}

void read_IRT(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    // while the peripheral is connected, turn the optic sensor on by writing a '1' to the characteristic
    IRTConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned int rawobj = (IRTValCharacteristic[0]) + (IRTValCharacteristic[1] * 256);
    unsigned int rawamb = (IRTValCharacteristic[2]) + (IRTValCharacteristic[3] * 256);
    IRTConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate final temperature values
    const float SCALE_LSB = 0.03125;
    int it = (int)( rawobj >> 2);
    float IRTo = ( (float)it) * SCALE_LSB;
    IRTo = ( (IRTo * 9.0) / 5.0 ) + 32.0; // convert to F - comment out to leave at C
    it = (int)(rawamb >> 2);
    float IRTa = (float)it * SCALE_LSB;
    IRTa = ( (IRTa * 9.0) / 5.0) + 32.0; // convert to F - comment out to leave at C
    // save into the structure
    SensorData.temd = IRTa;
    SensorData.temo = IRTo;
  }
  else {
    // insert not connected error signal here if you want
  }
}

void read_HUM(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    // while the peripheral is connected, turn the humidity sensor on by writing a '1' to the characteristic
    HUMConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned int rawtem = (HUMValCharacteristic[0]) + (HUMValCharacteristic[1] * 256);
    unsigned int rawhum = (HUMValCharacteristic[2]) + (HUMValCharacteristic[3] * 256);
    HUMConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate final temperature and relative humidity values
    float temp = (rawtem / 65536.0) * 165.0 - 40.0;
    temp = ((temp * 9.0) / 5.0) + 32.0; // convert to F - comment out to leave at C
    float hum = ((double)rawhum / 65536.0) * 100.0;
    // save into the structure
    SensorData.tem = temp;
    SensorData.hum = hum;
  }
  else {
    // insert not connected error signal here if you want
  }

}
// Write the data to the SD card
void write_data() {
  // kick the green LED on -
  digitalWrite(green_LED, HIGH);
  // open the file
  SDF = SD.open(fname, FILE_WRITE);
  if (!SDF) {
    // terminal error if we can't open the SD File
    t_error();
  }
  else {
    // write the comma-delimited data line
    // unixtime, temperature, humidity, barometric pressure, die temp, lux, object temp, die temp
    now = rtc.now();
    SDF.print(now.unixtime());
    SDF.print(",");
    SDF.print(SensorData.tem);
    SDF.print(",");
    SDF.print(SensorData.hum);
    SDF.print(",");
    SDF.print(SensorData.bp);
    SDF.print(",");
    SDF.print(SensorData.bptemp);
    SDF.print(",");
    SDF.print(SensorData.li);
    SDF.print(",");
    SDF.print(SensorData.temo);
    SDF.print(",");
    SDF.println(SensorData.temd);
    SDF.close();
  }
}

void t_error() {
  // flash both LEDs ad infinitum to alert user to
  // a terminal error - stays forever!
  while (1) {
    digitalWrite(red_LED, HIGH);     // turn the red LED on
    digitalWrite(green_LED, HIGH);   // turn the green LED on
    delay(500);                      // half second
    digitalWrite(red_LED, LOW);      // turn the red LED off
    digitalWrite(green_LED, LOW);    // turn the green LED off
    delay(500);                      // half second
  }
}

// fine

