// *****************************************************************************
// Arduino 101 data logger using TI SensorTag CC2650                          **
// Software to accompany the article:                                         **
// "Build an Arduino 101 Data Logger with the TI SensorTag"                   **
// *****************************************************************************
// This software is offered strictly as is with no warranties whatsoever.     **
// Use it at your own risk.                                                   **
// *****************************************************************************
// SensorTagTestv1.ino

#include <CurieBLE.h>
// Global structure to hold the sensor data
struct DATA {
  float tem;     // HDC1000 temperature F
  float hum;     // HDC1000 relative humidity %RH
  float bptemp;  // BMP 280 die temperature F
  float bp;      // BMP 280 barometric pressure in hectoPascals (1 hPa = 100 Pa)
  float li;      // OPT3001 lux
  float temd;    // TMP007 die temperature F
  float temo;    // TMP007 object temperature F
};

typedef struct DATA DATA;
DATA SensorData;

//period defines the length of time between measurements in milliseconds
//long period=600000L; // 10 minutes
long period = 30000L;  // .5 minute for testing

BLEDevice peripheral;

// BMP280
BLECharacteristic BPConCharacteristic;
BLECharacteristic BPValCharacteristic;
// OPT3001
BLECharacteristic OPTConCharacteristic;
BLECharacteristic OPTValCharacteristic;
// TMP007
BLECharacteristic IRTConCharacteristic;
BLECharacteristic IRTValCharacteristic;
// HDC1000
BLECharacteristic HUMConCharacteristic;
BLECharacteristic HUMValCharacteristic;

void setup() {
  Serial.begin(9600);
  while (!Serial);
  Serial.println("     *** SensorTag Test (v1.0) ***");
  Serial.println();
  // initialize the BLE hardware
  BLE.begin();
  BLE.scan();
}

void loop() {
  long lastMillis = 0;  // for period test
  long nowMillis = 0;   // for period test

  // check if a peripheral has been discovered
  peripheral = BLE.available();
  if (peripheral) {
    // discovered a peripheral, print out address and local name
    Serial.print("Found ");
    Serial.print(peripheral.address());
    Serial.print(" '");
    Serial.print(peripheral.localName());
    Serial.println("' ");
    if (peripheral.localName() == "CC2650 SensorTag") {
      // stop scanning
      BLE.stopScan();
      // connect to the peripheral
      Serial.print("Connecting to SensorTag ...");
      if (peripheral.connect()) {
        Serial.println("Connected");
        do_discovery(peripheral);
        subscribe_BP(peripheral);
        subscribe_OPT(peripheral);
        subscribe_IRT(peripheral);
        subscribe_HUM(peripheral);
        Serial.println("Reading sensors...");
        Serial.println();
      } else {
        Serial.println("Failed to connect!");
      }
    }
    // main while connected loop
    while (peripheral.connected()) {
      read_BP(peripheral);
      read_OPT(peripheral);
      read_IRT(peripheral);
      read_HUM(peripheral);
      print_data();
      lastMillis = millis();
      // stay here until the period is up
      while ( ( (nowMillis = millis()) - lastMillis) <= period);
    }
    // peripheral disconnected, start scanning again
    Serial.println("rescan...");
    BLE.scan();
  }
}

// discover the peripheral's attributes that we want
void do_discovery(BLEDevice peripheral) {
  // discover peripheral attributes for the barometric pressure sensor
  Serial.print("Discovering attributes for Barometric Pressure service ...");
  if (peripheral.discoverAttributesByService("F000AA40-0451-4000-B000-000000000000")) {
    Serial.println("discovered");
    BPConCharacteristic = peripheral.characteristic("F000AA42-0451-4000-B000-000000000000");
    BPValCharacteristic = peripheral.characteristic("F000AA41-0451-4000-B000-000000000000");
  }
  else  {
    Serial.println("ERROR: Barometric Pressure service discovery failed.");
    peripheral.disconnect();
    return;
  }
  // discover peripheral attributes for the optical sensor
  Serial.print("Discovering attributes for Luxometer service ...");
  if (peripheral.discoverAttributesByService("F000AA70-0451-4000-B000-000000000000")) {
    Serial.println("discovered");
    OPTConCharacteristic = peripheral.characteristic("F000AA72-0451-4000-B000-000000000000");
    OPTValCharacteristic = peripheral.characteristic("F000AA71-0451-4000-B000-000000000000");
  }
  else  {
    Serial.println("Error: Luxometer service discovery failed.");
    peripheral.disconnect();
    return;
  }
  // discover peripheral attributes for the infrared sensor
  Serial.print("Discovering attributes for Infrared service ...");
  if (peripheral.discoverAttributesByService("F000AA00-0451-4000-B000-000000000000")) {
    Serial.println("discovered");
    IRTConCharacteristic = peripheral.characteristic("F000AA02-0451-4000-B000-000000000000");
    IRTValCharacteristic = peripheral.characteristic("F000AA01-0451-4000-B000-000000000000");
  }
  else  {
    Serial.println("Error: Infrared service discovery failed.");
    peripheral.disconnect();
    return;
  }
  // discover peripheral attributes for the humidity sensor
  Serial.print("Discovering attributes for Humidity service ...");
  if (peripheral.discoverAttributesByService("F000AA20-0451-4000-B000-000000000000")) {
    Serial.println("discovered");
    HUMConCharacteristic = peripheral.characteristic("F000AA22-0451-4000-B000-000000000000");
    HUMValCharacteristic = peripheral.characteristic("F000AA21-0451-4000-B000-000000000000");
  }
  else  {
    Serial.println("Error: Humidity service discovery failed.");
    peripheral.disconnect();
    return;
  }
}

// subscribe to each peripheral service that we want
void subscribe_BP(BLEDevice peripheral)
{
  // subscribe to the barometric service
  Serial.print("Subscribing to the Barometric data service  ...");
  if (!BPValCharacteristic) {
    Serial.println("Error: barometric pressure data service characteristic not found!");
    peripheral.disconnect();
    return;
  } else if (!BPValCharacteristic.canSubscribe()) {
    Serial.println("Error: barometric pressure data service characteristic not subscribable!");
    peripheral.disconnect();
    return;
  } else if (!BPValCharacteristic.subscribe()) {
    Serial.println("Error: subscription failed!");
    peripheral.disconnect();
    return;
  } else {
    Serial.println("Subscribed to barometric pressure data service characteristic");
    delay(1000);
  }
}

void subscribe_OPT(BLEDevice peripheral) {
  // subscribe to the luxometer service
  Serial.print("Subscribing to the Luxometer data service  ...");
  if (!OPTValCharacteristic) {
    Serial.println("Error: luxometer data service characteristic not found!");
    peripheral.disconnect();
    return;
  } else if (!OPTValCharacteristic.canSubscribe()) {
    Serial.println("Error: luxometer data service characteristic not subscribable!");
    peripheral.disconnect();
    return;
  } else if (!OPTValCharacteristic.subscribe()) {
    Serial.println("Error: subscription failed!");
    peripheral.disconnect();
    return;
  } else {
    Serial.println("Subscribed to luxometer data service characteristic");
    delay(1000);
  }
}

void subscribe_IRT(BLEDevice peripheral) {
  // subscribe to the infrared service
  Serial.print("Subscribing to the Infrared data service  ...");
  if (!IRTValCharacteristic) {
    Serial.println("Error: infrared data service characteristic not found!");
    peripheral.disconnect();
    return;
  } else if (!IRTValCharacteristic.canSubscribe()) {
    Serial.println("Error: infrared data service characteristic not subscribable!");
    peripheral.disconnect();
    return;
  } else if (!IRTValCharacteristic.subscribe()) {
    Serial.println("Error: subscription failed!");
    peripheral.disconnect();
    return;
  } else {
    Serial.println("Subscribed to infrared data service characteristic");
    delay(1000);
  }
}

void subscribe_HUM(BLEDevice peripheral) {
  // subscribe to the humidity service
  Serial.print("Subscribing to the Humidity data service  ...");
  if (!HUMValCharacteristic) {
    Serial.println("Error: humidity data service characteristic not found!");
    peripheral.disconnect();
    return;
  } else if (!HUMValCharacteristic.canSubscribe()) {
    Serial.println("Error: humidity data service characteristic not subscribable!");
    peripheral.disconnect();
    return;
  } else if (!HUMValCharacteristic.subscribe()) {
    Serial.println("Error: subscription failed!");
    peripheral.disconnect();
    return;
  } else {
    Serial.println("Subscribed to humidity data service characteristic");
    delay(1000);
  }
}

// Sensor reads
void read_BP(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    BPConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned long rawbptemp = (BPValCharacteristic[2] * 65536) + (BPValCharacteristic[1] * 256) + BPValCharacteristic[0];
    unsigned int rawbp = (BPValCharacteristic[5] * 65536) + (BPValCharacteristic[4] * 256) + BPValCharacteristic[3];
    BPConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate temperature and pressure final values
    float bptemp = ((double)rawbptemp / 100.0);
    bptemp = ((bptemp * 9.0) / 5.0) + 32.0; // convert to F - comment out to leave at C
    float bp = ((double)rawbp / 100.0);
    // save into the structure
    SensorData.bp = bp;
    SensorData.bptemp = bptemp;
  }
  else {
    Serial.println(" *not connected* ");
  }
}

void read_OPT(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    // while the peripheral is connected, turn the optic sensor on by writing a '1' to the characteristic
    OPTConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned int rawlux = (OPTValCharacteristic[0]) + (OPTValCharacteristic[1] * 256);
    OPTConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate lux final value
    unsigned int m = rawlux & 0x0FFF;
    unsigned int e = (rawlux & 0xF000) >> 12;
    float lux = (m * (0.01 * pow(2.0, e)));
    // save into the structure
    SensorData.li = lux;
  }
  else {
    Serial.println(" *not connected* ");
  }
}

void read_IRT(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    // while the peripheral is connected, turn the optic sensor on by writing a '1' to the characteristic
    IRTConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned int rawobj = (IRTValCharacteristic[0]) + (IRTValCharacteristic[1] * 256);
    unsigned int rawamb = (IRTValCharacteristic[2]) + (IRTValCharacteristic[3] * 256);
    IRTConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate final temperature values
    const float SCALE_LSB = 0.03125;
    int it = (int)( rawobj >> 2);
    float IRTo = ( (float)it) * SCALE_LSB;
    IRTo = ( (IRTo * 9.0) / 5.0 ) + 32.0; // convert to F - comment out to leave at C
    it = (int)(rawamb >> 2);
    float IRTa = (float)it * SCALE_LSB;
    IRTa = ( (IRTa * 9.0) / 5.0) + 32.0; // convert to F - comment out to leave at C
    // save into the structure
    SensorData.temd = IRTa;
    SensorData.temo = IRTo;
  }
  else {
    Serial.println(" *not connected* ");
  }
}

void read_HUM(BLEDevice peripheral) {
  // must be subscribed
  if (peripheral.connected()) {
    // while the peripheral is connected, turn the humidity sensor on by writing a '1' to the characteristic
    HUMConCharacteristic.writeByte(0x01);
    delay(1200); // wait for the sensor to do a read
    unsigned int rawtem = (HUMValCharacteristic[0]) + (HUMValCharacteristic[1] * 256);
    unsigned int rawhum = (HUMValCharacteristic[2]) + (HUMValCharacteristic[3] * 256);
    HUMConCharacteristic.writeByte(0x00); // turn the sensor off
    // calculate final temperature and relative humidity values
    float temp = (rawtem / 65536.0) * 165.0 - 40.0;
    temp = ((temp * 9.0) / 5.0) + 32.0; // convert to F - comment out to leave at C
    float hum = ((double)rawhum / 65536.0) * 100.0;
    // save into the structure
    SensorData.tem = temp;
    SensorData.hum = hum;
  }
  else {
    Serial.println(" *not connected* ");
  }

}
// Print the data to the serial moniter
void print_data() {
  Serial.print("Temperature (F)=");
  Serial.print(SensorData.tem);
  Serial.print(" Humidity (%RH)=");
  Serial.print(SensorData.hum);
  Serial.print(" Barometric Pressure (hPa)=");
  Serial.print(SensorData.bp);
  Serial.print(" (die(F))=");
  Serial.print(SensorData.bptemp);
  Serial.print(" Light (lux)=");
  Serial.print(SensorData.li);
  Serial.print(" Object Temperature (F)=");
  Serial.print(SensorData.temo);
  Serial.print(" (die(F))=");
  Serial.println(SensorData.temd);
}




